/**
 * Created by jmoutard on 19/07/16.
 */

'use strict';
/*
 * Local Storage Service give data from local storage (HTML5) if not logged in and is capable of giving errors.
 * Errors are:
 *          - 10; No local storage (no HTML5 local storage),
 *          - 11; Error when adding dashboard (Here we just add a newly create dashboard),
 *          - 12; Dashboard not found (local storage stores just one dashboard),
 *          - 13; Can't remove the dashboard.
 *
 * For convenience error code from 10 to 19 belongs to local storage errors.
 * NB: If you want to create a new local dashboard and you already have one, you need to remove the one existing first before create a new one.
 */
angular.module('adf.electre')
    .service('localStorageService', [
        "$q", "storageErrorProvider",
        function ($q, storageErrorProvider) {
            /**
             * @description Return all stored dashboard from the server.
             * @returns {Promise}
             */
            function getAll () {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("dashboard");
                        try {
                            electre_storage = JSON.parse(electre_storage);
                            deferred.resolve([{id: "localDashboard",title:electre_storage.title}]);
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(12, "No dashboard found."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Return a specific stored dashboard from the server.
             * @param {string} id
             * @returns {Promise}
             */
            function get (id) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("dashboard");
                        try {
                            deferred.resolve(JSON.parse(electre_storage));
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(12, "No dashboard found."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Add a dashboard in the local storage.
             * @param {string} id
             * @param {object} data
             * @returns {Promise}
             */
            function add (id, data) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        if (!localStorage.getItem("dashboard")) {
                            try {
                                localStorage.setItem("dashboard", JSON.stringify(data));
                                deferred.resolve(true);
                            } catch (e) {
                                deferred.reject(new storageErrorProvider(11, "Can't add dashboard."));
                            }
                        } else if (id === "localDashboard") {
                            try {
                                localStorage.setItem("dashboard", JSON.stringify(data));
                                deferred.resolve(true);
                            } catch (e) {
                                deferred.reject(new storageErrorProvider(11, "Can't add dashboard."));
                            }
                        } else
                            deferred.reject(new storageErrorProvider(14, "A local dashboard already exist."));
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Remove a dashboard onto the server.
             * @param {string} id
             * @returns {Promise}
             */
            function remove (id) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        try {
                            localStorage.removeItem("dashboard");
                            deferred.resolve(true);
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(13, "Can't remove dashboard."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            return { getAll: getAll, get: get, add: add, remove: remove};
        }
    ]);